//************************************************************
//
// BPNGLoggerDetector.cc
//
/** @file BPNGLoggerDetector.cc
 *
 * @brief Description
 *
 * Description
 */
//************************************************************

#include <ctime>
#include <map>

#ifndef _MSC_VER
#include <tmlib-config.h>
#include <BPNGClient.hh>
#endif
#include "BPNGLoggerDetector.hh"

using namespace std;


// Returns a vector of BP2Devices, search the network for searchTimeOut seconds
vector<BPNGDevice> BPNGLoggerDetector::getLoggerList(unsigned searchTimeOut)
{
    m_loggerList.clear();

#ifdef _MSC_VER
    IBPNGClient *client = getBPNGClient();
#else
    IBPNGClient *client = new BPNGClient();
#endif

    client->addListener(this);

    time_t start = time(0);

    if (searchTimeOut == 0)
    {
        // scan only once
        client->scanNetworkForLogger();
    }
    else
    {
        while (time(0) - start < searchTimeOut)
            client->scanNetworkForLogger();
    }

    client->release();
    return m_loggerList;
}

vector<TSLCluster> BPNGLoggerDetector::getTSLs(vector<BPNGDevice> loggersInNetwork)
{
    map<int, TSLCluster> tsls;

    for (size_t i = 0; i < loggersInNetwork.size(); ++i)
    {
		BPNGDevice device = loggersInNetwork[i];
        if (device.tslNetworkId != -1)
        {
            if (tsls.find(device.tslNetworkId) == tsls.end())
            {
                tsls.insert(make_pair(device.tslNetworkId, TSLCluster(device)));
            }
            else
            {
                tsls[device.tslNetworkId].addDevice(device);
            }
        }
    }

    vector<TSLCluster> retVal;

	map<int, TSLCluster>::iterator iter;
	for (iter = tsls.begin(); iter != tsls.end(); ++iter)
    {
        retVal.push_back(iter->second);
    }

    return retVal;
}

// listener function for observing blue PiraT 2 devices in network
void WINAPI BPNGLoggerDetector::onBPNGDeviceDetected(OnlineLoggerInfo *info)
{
    BPNGDevice newDevice(info);

    bool alreadyIn = false;
    for (size_t i = 0; i < m_loggerList.size(); ++i)
    {
        if (m_loggerList[i].mainboardNr == newDevice.mainboardNr)
            alreadyIn = true;
    }

    if (!alreadyIn)
        m_loggerList.push_back(newDevice);
}

void WINAPI BPNGLoggerDetector::onBPNGDeviceDisappeared(OnlineLoggerInfo *info)
{
    BPNGDevice device(info);
    for (size_t i = 0; i < m_loggerList.size(); ++i)
    {
        if (m_loggerList[i].mainboardNr == device.mainboardNr)
        {
            m_loggerList.erase(m_loggerList.begin() + i);
            break;
        }
    }
}

void WINAPI BPNGLoggerDetector::onBPNGDeviceStateChange(OnlineLoggerInfo *info)
{
    BPNGDevice device(info);
    for (size_t i = 0; i < m_loggerList.size(); ++i)
    {
        if (m_loggerList[i].mainboardNr == device.mainboardNr)
        {
            m_loggerList[i] = BPNGDevice(info);
            break;
        }
    }
}
